// Macro used in the Java Lecture of the 2016 STAR Global Conference
// @author: paco.ezquerra@cd-adapco.com
package macro;

import java.io.File;
import star.base.neo.*;
import star.base.report.*;
import star.cadmodeler.*;
import star.common.*;
import star.coremodule.actions.FileChooserAction;
import star.coremodule.ui.StarFileOpenChooser;
import star.flow.*;
import star.keturb.*;
import star.material.SingleComponentGasModel;
import star.meshing.*;
import star.metrics.TwoDimensionalModel;
import star.prismmesher.*;
import star.resurfacer.SurfaceGrowthRate;
import star.segregatedenergy.SegregatedFluidTemperatureModel;
import star.segregatedflow.SegregatedFlowModel;
import star.turbulence.RansTurbulenceModel;
import star.turbulence.TurbulentModel;
import star.vis.*;

abstract class Workflow extends StarMacro {

    public void setFile(String extension) {

        final String ext = extension;

        (new FileChooserAction() {

            @Override
            protected NeoFileChooser createFileChooser() {
                return new StarFileOpenChooser(new String[]{ext}, new String[]{"Please choose files with extension " + ext},
                        new File(resolvePath(".")), sim);
            }

            @Override
            protected void fileSelected(File f) {
                sim.println("selected File is " + f.getName());
                sf = f;
                PartName = f.getName();
                PartName = PartName.substring(0, PartName.lastIndexOf("."));

            }

        }).performAction();

    }

    public void generatePart(double radius) {

        CadModel cadModel_0
                = sim.get(SolidModelManager.class).createSolidModel();

        LabCoordinateSystem labCoordinateSystem_0
                = sim.getCoordinateSystemManager().getLabCoordinateSystem();

        cadModel_0.getFeatureManager().create3DSketches(sf.getAbsolutePath(), labCoordinateSystem_0, true, false);

        Sketch3D sketch3D_0
                = ((Sketch3D) cadModel_0.getFeature("3DCurve 1"));

        ExtrusionMerge extrusionMerge_0
                = cadModel_0.getFeatureManager().createExtrusionMerge(sketch3D_0);

        Units units_0
                = sim.getUnitsManager().getPreferredUnits(new IntVector(new int[]{0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}));

        extrusionMerge_0.setDirectionOption(2);

        extrusionMerge_0.setExtrudedBodyTypeOption(0);

        extrusionMerge_0.setDistanceOption(0);

        extrusionMerge_0.setCoordinateSystemOption(1);

        extrusionMerge_0.setDraftOption(0);

        extrusionMerge_0.setCoordinateSystem(labCoordinateSystem_0);

        CadModelCoordinate cadModelCoordinate_0
                = extrusionMerge_0.getDirectionAxis();

        cadModelCoordinate_0.setCoordinateSystem(labCoordinateSystem_0);

        cadModelCoordinate_0.setCoordinate(units_0, units_0, units_0, new DoubleVector(new double[]{0.0, 0.0, 1.0}));

        extrusionMerge_0.setFace(null);

        extrusionMerge_0.setBody(null);

        extrusionMerge_0.setSketch(sketch3D_0);

        extrusionMerge_0.setSketch(sketch3D_0);

        extrusionMerge_0.setPostOption(1);

        extrusionMerge_0.setExtrusionOption(0);

        BodyNameRefManager bodyNameRefManager_0
                = extrusionMerge_0.getInteractionBodies();

        bodyNameRefManager_0.setBodies(new NeoObjectVector(new Object[]{}));

        extrusionMerge_0.setInteractingSelectedBodies(false);

        extrusionMerge_0.markFeatureForEdit();

        cadModel_0.getFeatureManager().execute(extrusionMerge_0);

        LineSketchPrimitive3D lineSketchPrimitive3D_0
                = ((LineSketchPrimitive3D) sketch3D_0.getSketchPrimitive3D("Line 1"));

        star.cadmodeler.Body cadmodelerBody_0
                = ((star.cadmodeler.Body) extrusionMerge_0.getBody(lineSketchPrimitive3D_0));

        cadmodelerBody_0.setPresentationName("Airfoil");

        CanonicalSketchPlane canonicalSketchPlane_0
                = ((CanonicalSketchPlane) cadModel_0.getFeature("XY"));

        Sketch sketch_0
                = cadModel_0.getFeatureManager().createSketch(canonicalSketchPlane_0);

        cadModel_0.getFeatureManager().startSketchEdit(sketch_0);

        CircleSketchPrimitive circleSketchPrimitive_0
                = sketch_0.createCircle(new DoubleVector(new double[]{0.0, 0.0}), 0.06403124237432849);

        sketch_0.setCircleRadius(circleSketchPrimitive_0, radius);

        sketch_0.markFeatureForEdit();

        cadModel_0.getFeatureManager().stopSketchEdit(sketch_0, true);

        cadModel_0.getFeatureManager().rollForwardToEnd();

        ExtrusionMerge extrusionMerge_1
                = cadModel_0.getFeatureManager().createExtrusionMerge(sketch_0);

        extrusionMerge_1.setDirectionOption(0);

        extrusionMerge_1.setExtrudedBodyTypeOption(0);

        extrusionMerge_1.setDistanceOption(0);

        extrusionMerge_1.setCoordinateSystemOption(0);

        extrusionMerge_1.setDraftOption(0);

        extrusionMerge_1.setCoordinateSystem(labCoordinateSystem_0);

        CadModelCoordinate cadModelCoordinate_1
                = extrusionMerge_1.getDirectionAxis();

        cadModelCoordinate_1.setCoordinateSystem(labCoordinateSystem_0);

        cadModelCoordinate_1.setCoordinate(units_0, units_0, units_0, new DoubleVector(new double[]{0.0, 0.0, 1.0}));

        extrusionMerge_1.setFace(null);

        extrusionMerge_1.setBody(null);

        extrusionMerge_1.setSketch(sketch_0);

        extrusionMerge_1.setSketch(sketch_0);

        extrusionMerge_1.setPostOption(0);

        extrusionMerge_1.setExtrusionOption(0);

        BodyNameRefManager bodyNameRefManager_1
                = extrusionMerge_1.getInteractionBodies();

        bodyNameRefManager_1.setBodies(new NeoObjectVector(new Object[]{}));

        extrusionMerge_1.setInteractingSelectedBodies(false);

        extrusionMerge_1.markFeatureForEdit();

        cadModel_0.getFeatureManager().execute(extrusionMerge_1);

        star.cadmodeler.Body cadmodelerBody_1
                = ((star.cadmodeler.Body) extrusionMerge_1.getBody(circleSketchPrimitive_0));

        cadmodelerBody_1.setPresentationName("Disk");

        SubtractBodiesFeature subtractBodiesFeature_0
                = cadModel_0.getFeatureManager().createSubtractBodies();

        BodyNameRefManager bodyNameRefManager_2
                = subtractBodiesFeature_0.getTargetBodyGroup();

        bodyNameRefManager_2.setBodies(new NeoObjectVector(new Object[]{cadmodelerBody_1}));

        BodyNameRefManager bodyNameRefManager_3
                = subtractBodiesFeature_0.getToolBodyGroup();

        bodyNameRefManager_3.setBodies(new NeoObjectVector(new Object[]{cadmodelerBody_0}));

        subtractBodiesFeature_0.setKeepToolBodies(false);

        subtractBodiesFeature_0.setImprint(false);

        subtractBodiesFeature_0.setImprintOption(0);

        subtractBodiesFeature_0.setTransferFaceNames(true);

        subtractBodiesFeature_0.setTransferBodyNames(false);

        subtractBodiesFeature_0.markFeatureForEdit();

        cadModel_0.getFeatureManager().execute(subtractBodiesFeature_0);

        cadmodelerBody_1.setPresentationName(PartName);

        sim.get(SolidModelManager.class).endEditCadModel(cadModel_0);

        cadModel_0.createParts(new NeoObjectVector(new Object[]{cadmodelerBody_1}), "SharpEdges", 30.0, 2, true, 1.0E-5);

    }

    public void mesh2D() {

        SolidModelPart solidModelPart_0
                = ((SolidModelPart) sim.get(SimulationPartManager.class).getPart(PartName));

        // Split
        PartSurface partSurface_0
                = ((PartSurface) solidModelPart_0.getPartSurfaceManager().getPartSurface("Default"));

        solidModelPart_0.getPartSurfaceManager().splitPartSurfacesByAngle(new NeoObjectVector(new Object[]{partSurface_0}), 89.0);

        PartSurface partSurface_4
                = ((PartSurface) solidModelPart_0.getPartSurfaceManager().getPartSurface("Default"));

        partSurface_4.setPresentationName("Environment");

        PartSurface partSurface_1
                = ((PartSurface) solidModelPart_0.getPartSurfaceManager().getPartSurface("Default 2"));

        partSurface_1.setPresentationName("Z+");

        PartSurface partSurface_2
                = ((PartSurface) solidModelPart_0.getPartSurfaceManager().getPartSurface("Default 3"));

        partSurface_2.setPresentationName("Z-");

        PartSurface partSurface_3
                = ((PartSurface) solidModelPart_0.getPartSurfaceManager().getPartSurface("Default 4"));

        partSurface_3.setPresentationName("Airfoil");

        sim.getRegionManager().newRegionsFromParts(new NeoObjectVector(new Object[]{solidModelPart_0}), "OneRegionPerPart", null, "OneBoundaryPerPartSurface", null, "OneFeatureCurvePerPartCurve", null, true);

        Region region_0
                = sim.getRegionManager().getRegion(PartName);

        Boundary boundary_0
                = region_0.getBoundaryManager().getBoundary("Environment");

        boundary_0.setBoundaryType(FreeStreamBoundary.class);

        PrepareFor2dOperation prepareFor2dOperation_0
                = (PrepareFor2dOperation) sim.get(MeshOperationManager.class).createPrepareFor2dOperation(new NeoObjectVector(new Object[]{solidModelPart_0}));

        prepareFor2dOperation_0.execute();

        AutoMeshOperation2d autoMeshOperation2d_0
                = sim.get(MeshOperationManager.class).createAutoMeshOperation2d(new StringVector(new String[]{"star.twodmesher.DualAutoMesher2d", "star.prismmesher.PrismAutoMesher"}), new NeoObjectVector(new Object[]{solidModelPart_0}));

        PrismAutoMesher prismAutoMesher_0
                = ((PrismAutoMesher) autoMeshOperation2d_0.getMeshers().getObject("Prism Layer Mesher"));

        prismAutoMesher_0.getPrismStretchingOption().setSelected(PrismStretchingOption.Type.WALL_THICKNESS);

        autoMeshOperation2d_0.getDefaultValues().get(BaseSize.class).setValue(0.1);

        SurfaceCurvature surfaceCurvature_0
                = autoMeshOperation2d_0.getDefaultValues().get(SurfaceCurvature.class);

        SurfaceCurvatureNumPts surfaceCurvatureNumPts_0
                = surfaceCurvature_0.getSurfaceCurvatureNumPts();

        surfaceCurvatureNumPts_0.setNumPointsAroundCircle(144.0);

        SurfaceGrowthRate surfaceGrowthRate_0
                = autoMeshOperation2d_0.getDefaultValues().get(SurfaceGrowthRate.class);

        surfaceGrowthRate_0.setGrowthRate(1.06);

        NumPrismLayers numPrismLayers_0
                = autoMeshOperation2d_0.getDefaultValues().get(NumPrismLayers.class);

        numPrismLayers_0.setNumLayers(8);

        Units units_0
                = ((Units) sim.getUnitsManager().getObject("mm"));

        autoMeshOperation2d_0.getDefaultValues().get(PrismWallThickness.class).setUnits(units_0);

        autoMeshOperation2d_0.getDefaultValues().get(PrismWallThickness.class).setValue(1.0);

        PrismThickness prismThickness_0
                = autoMeshOperation2d_0.getDefaultValues().get(PrismThickness.class);

        prismThickness_0.getRelativeOrAbsoluteOption().setSelected(RelativeOrAbsoluteOption.Type.ABSOLUTE);

        GenericAbsoluteSize genericAbsoluteSize_0
                = ((GenericAbsoluteSize) prismThickness_0.getAbsoluteSize());

        genericAbsoluteSize_0.getValue().setValue(0.0125);

        SurfaceCustomMeshControl surfaceCustomMeshControl_0
                = autoMeshOperation2d_0.getCustomMeshControls().createSurfaceControl();

        surfaceCustomMeshControl_0.getGeometryObjects().setQuery(null);

        surfaceCustomMeshControl_0.getGeometryObjects().setObjects(partSurface_3);

        surfaceCustomMeshControl_0.getCustomConditions().get(PartsTargetSurfaceSizeOption.class).setSelected(PartsTargetSurfaceSizeOption.Type.CUSTOM);

        PartsTargetSurfaceSize partsTargetSurfaceSize_1
                = surfaceCustomMeshControl_0.getCustomValues().get(PartsTargetSurfaceSize.class);

        GenericRelativeSize genericRelativeSize_2
                = ((GenericRelativeSize) partsTargetSurfaceSize_1.getRelativeSize());

        genericRelativeSize_2.setPercentage(10.0);

        PartsTargetSurfaceSize partsTargetSurfaceSize_0
                = autoMeshOperation2d_0.getDefaultValues().get(PartsTargetSurfaceSize.class);

        GenericRelativeSize genericRelativeSize_0
                = ((GenericRelativeSize) partsTargetSurfaceSize_0.getRelativeSize());

        genericRelativeSize_0.setPercentage(200.0);

    // mesh It
        MeshPipelineController meshPipelineController_0
                = sim.get(MeshPipelineController.class);

        meshPipelineController_0.generateVolumeMesh();

    }

    public void defineUserFieldFunctions(Double angle) {
        
        UserFieldFunction userFieldFunction_0
                = sim.getFieldFunctionManager().createFieldFunction();

        userFieldFunction_0.getTypeOption().setSelected(FieldFunctionTypeOption.Type.SCALAR);

        userFieldFunction_0.setPresentationName("_AngleF_" + PartName);

        userFieldFunction_0.setFunctionName("AngleF_" + PartName);

        userFieldFunction_0.setDimensionsVector(new IntVector(new int[]{0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}));

        userFieldFunction_0.setDefinition(angle.toString() + "*<1 deg>");

        Units units_0
                = ((Units) sim.getUnitsManager().getObject("deg"));

        units_0.setPreferred(true);

        UserFieldFunction userFieldFunction_1
                = sim.getFieldFunctionManager().createFieldFunction();

        userFieldFunction_1.getTypeOption().setSelected(FieldFunctionTypeOption.Type.VECTOR);

        userFieldFunction_1.setPresentationName("_FlowDirection_" + PartName);

        userFieldFunction_1.setFunctionName("FlowDirection_" + PartName);

        userFieldFunction_1.setDefinition("[ cos($AngleF_" + PartName + "), sin($AngleF_" + PartName + "), 0 ]");

        UserFieldFunction userFieldFunction_2
                = sim.getFieldFunctionManager().createFieldFunction();

        userFieldFunction_2.getTypeOption().setSelected(FieldFunctionTypeOption.Type.VECTOR);

        userFieldFunction_2.setPresentationName("_Yaxis_" + PartName);

        userFieldFunction_2.setFunctionName("Yaxis_" + PartName);

        userFieldFunction_2.setDefinition("[ -1*sin($AngleF_" + PartName + "), cos($AngleF_" + PartName + "), 0 ]");

    }

    public void setPhysics() {

        sim.getContinuumManager().createContinuum(PhysicsContinuum.class);

        PhysicsContinuum physicsContinuum_0
                = ((PhysicsContinuum) sim.getContinuumManager().getContinuum("Physics 2"));

        physicsContinuum_0.setPresentationName("Physics " + PartName);

        physicsContinuum_0.enable(TwoDimensionalModel.class);

        physicsContinuum_0.enable(SteadyModel.class);

        physicsContinuum_0.enable(SingleComponentGasModel.class);

        physicsContinuum_0.enable(SegregatedFlowModel.class);

        physicsContinuum_0.enable(ConstantDensityModel.class);

        ConstantDensityModel constantDensityModel_0
                = physicsContinuum_0.getModelManager().getModel(ConstantDensityModel.class);

        physicsContinuum_0.disableModel(constantDensityModel_0);

        physicsContinuum_0.enable(IdealGasModel.class);

        physicsContinuum_0.enable(SegregatedFluidTemperatureModel.class);

        physicsContinuum_0.enable(TurbulentModel.class);

        physicsContinuum_0.enable(RansTurbulenceModel.class);

        physicsContinuum_0.enable(KEpsilonTurbulence.class);

        physicsContinuum_0.enable(RkeTwoLayerTurbModel.class);

        physicsContinuum_0.enable(KeTwoLayerAllYplusWallTreatment.class);

        VelocityProfile velocityProfile_0
                = physicsContinuum_0.getInitialConditions().get(VelocityProfile.class);

        velocityProfile_0.setMethod(CompositeVectorProfileMethod.class);

        ScalarProfile scalarProfile_0
                = velocityProfile_0.getMethod(CompositeVectorProfileMethod.class).getProfile(0);

        scalarProfile_0.getMethod(ConstantScalarProfileMethod.class).getQuantity().setDefinition("34.736*cos($AngleF_" + PartName + ")");

        ScalarProfile scalarProfile_1
                = velocityProfile_0.getMethod(CompositeVectorProfileMethod.class).getProfile(1);

        scalarProfile_1.getMethod(ConstantScalarProfileMethod.class).getQuantity().setDefinition("34.736*sin($AngleF_" + PartName + ")");
    }

    public void defineBoundaryConditions() {
        Region region_0
                = sim.getRegionManager().getRegion(PartName);

        PhysicsContinuum physicsContinuum_0
                = ((PhysicsContinuum) sim.getContinuumManager().getContinuum("Physics " + PartName));

        physicsContinuum_0.add(region_0);

        Boundary boundary_0
                = region_0.getBoundaryManager().getBoundary("Environment");

        boundary_0.setBoundaryType(FreeStreamBoundary.class);

        FlowDirectionProfile flowDirectionProfile_0
                = boundary_0.getValues().get(FlowDirectionProfile.class);

        flowDirectionProfile_0.setMethod(FunctionVectorProfileMethod.class);

        UserFieldFunction userFieldFunction_1
                = ((UserFieldFunction) sim.getFieldFunctionManager().getFunction("FlowDirection_" + PartName));

        flowDirectionProfile_0.getMethod(FunctionVectorProfileMethod.class).setFieldFunction(userFieldFunction_1);

        MachNumberProfile machNumberProfile_0
                = boundary_0.getValues().get(MachNumberProfile.class);

        machNumberProfile_0.getMethod(ConstantScalarProfileMethod.class).getQuantity().setValue(0.1);
    }

    public void configurePostProcessing() {

        sim.getSceneManager().createScalarScene("Scalar Scene", "Outline", "Scalar");

        Scene scene_0
                = sim.getSceneManager().getScene("Scalar Scene 1");

        scene_0.initializeAndWait();

        scene_0.setPresentationName(PartName + " :: Angle, Lift & Velocity Magnitude");

        PartDisplayer partDisplayer_0
                = ((PartDisplayer) scene_0.getDisplayerManager().getDisplayer("Outline 1"));

        partDisplayer_0.initialize();

        ScalarDisplayer scalarDisplayer_0
                = ((ScalarDisplayer) scene_0.getDisplayerManager().getDisplayer("Scalar 1"));

        Region region_0
                = sim.getRegionManager().getRegion(PartName);

        scalarDisplayer_0.getInputParts().setObjects(region_0);

        scalarDisplayer_0.initialize();

        scalarDisplayer_0.setPresentationName("Velocity Magnitude");

        scene_0.open(true);

        CurrentView currentView_0
                = scene_0.getCurrentView();

        PrimitiveFieldFunction primitiveFieldFunction_0
                = ((PrimitiveFieldFunction) sim.getFieldFunctionManager().getFunction("Velocity"));

        VectorMagnitudeFieldFunction vectorMagnitudeFieldFunction_0
                = ((VectorMagnitudeFieldFunction) primitiveFieldFunction_0.getMagnitudeFunction());

        scalarDisplayer_0.getScalarDisplayQuantity().setFieldFunction(vectorMagnitudeFieldFunction_0);

        Legend legend_0
                = scalarDisplayer_0.getLegend();

        PredefinedLookupTable predefinedLookupTable_0
                = ((PredefinedLookupTable) sim.get(LookupTableManager.class).getObject("gray-darkred"));

        legend_0.setLookupTable(predefinedLookupTable_0);

        scalarDisplayer_0.setFillMode(1);

        // Create Reports
        ForceReport forceReport_0
                = sim.getReportManager().createReport(ForceReport.class);

        forceReport_0.setPresentationName("Lift_" + PartName);

        forceReport_0.getDirection().setDefinition("[-1*sin($AngleF_" + PartName + "), cos($AngleF_" + PartName + "), 0.0]");

        Boundary boundary_1
                = region_0.getBoundaryManager().getBoundary("Airfoil");

        forceReport_0.getParts().setObjects(boundary_1);

        sim.getMonitorManager().createMonitorAndPlot(new NeoObjectVector(new Object[]{forceReport_0}), true, "%1$s Plot");

        ReportMonitor reportMonitor_0
                = ((ReportMonitor) sim.getMonitorManager().getMonitor("Lift_" + PartName + " Monitor"));

        MonitorPlot monitorPlot_0
                = sim.getPlotManager().createMonitorPlot(new NeoObjectVector(new Object[]{reportMonitor_0}), "Lift_" + PartName + " Monitor Plot");

        ReportAnnotation reportAnnotation_1
                = sim.getAnnotationManager().createReportAnnotation(forceReport_0);

        FixedAspectAnnotationProp fixedAspectAnnotationProp_0
                = (FixedAspectAnnotationProp) scene_0.getAnnotationPropManager().createPropForAnnotation(reportAnnotation_1);

        fixedAspectAnnotationProp_0.setPosition(new DoubleVector(new double[]{0.0836748633879782, 0.8017107309486775, 0.0}));

        MaxReport maxReport_0
                = sim.getReportManager().createReport(MaxReport.class);

        maxReport_0.setPresentationName("Angle_" + PartName);

        Boundary boundary_2
                = region_0.getBoundaryManager().getBoundary("Environment");

        maxReport_0.getParts().setObjects(region_0, boundary_1, boundary_2);

        UserFieldFunction userFieldFunction_0
                = ((UserFieldFunction) sim.getFieldFunctionManager().getFunction("AngleF_" + PartName));

        maxReport_0.setScalar(userFieldFunction_0);

        ReportAnnotation reportAnnotation_2
                = sim.getAnnotationManager().createReportAnnotation(maxReport_0);

        FixedAspectAnnotationProp fixedAspectAnnotationProp_1
                = (FixedAspectAnnotationProp) scene_0.getAnnotationPropManager().createPropForAnnotation(reportAnnotation_2);

        fixedAspectAnnotationProp_0.setPosition(new DoubleVector(new double[]{0.04711060074048901, 0.5721546237331788, 0.0}));

        fixedAspectAnnotationProp_1.setPosition(new DoubleVector(new double[]{0.04486655416395652, 0.45412776528353654, 0.0}));

        //Transform to apply to Scene
        SimpleTransform simpleTransform_0
                = sim.getTransformManager().createSimpleTransform("Simple Transform");

        simpleTransform_0.setPresentationName("Rotation_" + PartName);

        Units units_0
                = sim.getUnitsManager().getInternalUnits(new IntVector(new int[]{0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}));

        simpleTransform_0.getRotationAngleQuantity().setUnits(units_0);

        simpleTransform_0.getRotationAngleQuantity().setDefinition("-1*$AngleF_" + PartName);

        scalarDisplayer_0.setVisTransform(simpleTransform_0);

        // Clip Velocity Range to [0,60]
        scalarDisplayer_0.getScalarDisplayQuantity().setRange(new DoubleVector(new double[]{0.0, 60.0}));

        scalarDisplayer_0.getScalarDisplayQuantity().setClip(0);

        partDisplayer_0.setVisibilityOverrideMode(2);

        // Stopping Criteria
        StepStoppingCriterion stepStoppingCriterion_0
                = ((StepStoppingCriterion) sim.getSolverStoppingCriterionManager().getSolverStoppingCriterion("Maximum Steps"));

        stepStoppingCriterion_0.setMaximumNumberSteps(10000);

        // Plot Anotation
        MonitorDataSet monitorDataSet_0
                = ((MonitorDataSet) monitorPlot_0.getDataSetManager().getDataSet("Lift_" + PartName + " Monitor"));

        LineStyle lineStyle_0
                = monitorDataSet_0.getLineStyle();

        lineStyle_0.setWidth(3);

        Axes axes_0
                = monitorPlot_0.getAxes();

        Axis axis_0
                = axes_0.getYAxis();

        axis_0.setMaximum(1500.0);

        axis_0.setMinimum(-1500.0);

        MultiColLegend multiColLegend_0
                = monitorPlot_0.getLegend();

        multiColLegend_0.setRelativePosition(0.23537415266036987, 0.37603306770324707);

        PlotImage plotImage_0
                = sim.getAnnotationManager().createPlotImage();

        plotImage_0.setPlot(monitorPlot_0);

        SceneImageAnnotationProp sceneImageAnnotationProp_0
                = (SceneImageAnnotationProp) scene_0.getAnnotationPropManager().createPropForAnnotation(plotImage_0);

        sceneImageAnnotationProp_0.setWidth(0.4);

        sceneImageAnnotationProp_0.setHeight(0.3);

        sceneImageAnnotationProp_0.setPosition(new DoubleVector(new double[]{-0.014334470989761094, 0.09379727685325268, 0.0}));

        currentView_0.setInput(new DoubleVector(new double[]{0.5680665840309178, -0.18546181938687678, -0.009084259103267378}), new DoubleVector(new double[]{0.5680665840309178, -0.18546181938687678, 4.776647533346684}), new DoubleVector(new double[]{0.0, 1.0, 0.0}), 1.2493266957098685, 0);

        partDisplayer_0.setVisibilityOverrideMode(0);

        partDisplayer_0.setVisTransform(simpleTransform_0);

        partDisplayer_0.setColorMode(0);

        partDisplayer_0.setMeshColor(new DoubleVector(new double[]{1.0, 1.0, 1.0}));

        scalarDisplayer_0.setFillMode(5);

        CurrentView currentView_1
                = scene_0.getCurrentView();

        currentView_1.setInput(new DoubleVector(new double[]{0.6124724533516056, 0.03286703810650201, -0.015565906594722456}), new DoubleVector(new double[]{0.6124724533516056, 0.03286703810650201, 4.776647533346684}), new DoubleVector(new double[]{0.0, 1.0, 0.0}), 1.2510187452426131, 0);

        scene_0.setBackgroundColorMode(0);

        SolidBackgroundColor solidBackgroundColor_0
                = scene_0.getSolidBackgroundColor();

        solidBackgroundColor_0.setColor(new DoubleVector(new double[]{1.0, 0.20000000298023224, 0.20000000298023224}));

        partDisplayer_0.getInputParts().setObjects(region_0);

        partDisplayer_0.setMeshColor(new DoubleVector(new double[]{0.9900000095367432, 1.0, 0.9399999976158142}));

        partDisplayer_0.setLineWidth(3.0);

    }

    public void saveSim(String folder) {
        sim.saveState(resolvePath(folder + PartName + ".sim"));

    }

    public void changeAngle(Double angle) {
        UserFieldFunction uFF = ((UserFieldFunction) sim.getFieldFunctionManager().getFunction("AngleF_" + PartName));
        uFF.setDefinition(angle.toString() + "*<1 deg>");
    }

    public double getLift() {
        ForceReport fr = ((ForceReport) sim.getReportManager().getReport("Lift_" + PartName));
        return fr.getReportMonitorValue();
    }

    public void display(String output) {
        sim.println(output);
    }

    public File getFile() {
        return sf;
    }

    public void setPartName(String PartName) {
        this.PartName = PartName;
    }

    public Simulation sim;
    private File sf;
    public String PartName;

}
